<?php class FormulatrixImport {

    /**
     * @param $parameterName
     * @param $parameters
     * @throws BadRequestException
     */
    public static function require($parameterName, $parameters){
        if(!isset($parameters[$parameterName])){
            throw new BadRequestException('FormulatrixImport: Parameter '.$parameterName.' is required');
        }
    }

    const USERNAME='formulatriximporter';
    const USERFULLNAME='Formulatrix Importer';

    public static function getUsername(){
        return static::USERNAME;
    }

    public static function getUserFullName(){
        return static::USERFULLNAME;
    }

    const SCORING_SYSTEM_NAME="Formulatrix";

    /**
     * @return array|mixed
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function getScores()
    {
        return static::getScoresWithScoringSystemName(static::SCORING_SYSTEM_NAME);
    }
    public static function getScoresWithScoringSystemName($scoringSystemName){
        $scoringSystem=database::queryGetOne(
            'SELECT id FROM crystalscoringsystem WHERE name=:name',
            array(
                ':name'=>$scoringSystemName
            ));
        if(!$scoringSystem){ return array(); }
        $scores=database::queryGetAll('SELECT id,label,scoreindex,hotkey,color FROM crystalscore WHERE crystalscoringsystemid=:id ORDER BY scoreindex', array(':id'=>$scoringSystem['id']));
        return $scores['rows'];
    }

    public static function createScores($params){
        return static::createScoresWithScoringSystemName($params, static::SCORING_SYSTEM_NAME);
    }

    /**
     * Does the actual work of createScores. We split it out like this so we can test the code doing the actual
     * work - testing creation with a pre-existing "Formulatrix" scoring system will fail.
     * @param $params
     * @param $scoringSystemName
     * @return array|mixed
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function createScoresWithScoringSystemName($params, $scoringSystemName){
        return $params;
        static::require('scores', $params);
        session::becomeAdmin();
        $scoringSystem=crystalscoringsystem::getByName($scoringSystemName);
        if(!$scoringSystem){
            $scoringSystem=crystalscoringsystem::create(array(
                'name'=>$scoringSystemName,
                'iscurrent'=>1
            ));
            $scoringSystemId=$scoringSystem['created']['id'];
            foreach ($params['scores'] as $score) {
                $color=substr("000000".dechex(1*$score['color']),-6);
                $index=$score['hotkey'];
                $request=array(
                    'crystalscoringsystemid'=>$scoringSystemId,
                    'scoreindex'=>$index,
                    'hotkey'=>$index,
                    'color'=>$color,
                    'label'=>$score['name'],
                    'name'=>$scoringSystemName.'_'.$score['name'],
                );
                crystalscore::create($request);
            }
        } else {
            $scoringSystemId=$scoringSystem['id'];
        }
        session::revokeAdmin();
        return array(
            'id'=>$scoringSystemId,
            'name'=>$scoringSystemName,
            'scores'=>static::getScores()
        );
    }

    /**
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     * @return array containing keys "fxImagingTaskId" and "numImages" (how many images are registered to this task in IceBear)
     */
    public static function getLatestImagingTask(){
        return static::getImageCountTotalAndImagedTime();
    }

    /**
     * @return int[]
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function getImagingTaskByFormulatrixId($params){
        static::require('id', $params);
        return static::getImageCountTotalAndImagedTime($params['id']);
    }

    /**
     * @param $fxImagingTaskId
     * @return int[]
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
    private static function getImageCountTotalAndImagedTime($fxImagingTaskId=false){
        session::becomeAdmin();
        $ret=array(
            "fxImagingTaskId"=>0,
            "dateTime"=>0,
            "numImages"=>0
        );
        $stmt='SELECT ims.manufacturerdatabaseid AS manufacturerdatabaseid, ims.imageddatetime AS imageddatetime
                FROM imagingsession AS ims, imager AS im
                WHERE ims.imagerid=im.id AND [[CONDITION]]
                ORDER BY ims.imageddatetime DESC';
        if(!$fxImagingTaskId){
            //No ImagingTask ID specified. Get the Fx ID of the latest Formulatrix imagingsession.)
            $stmt=str_replace('[[CONDITION]]','im.manufacturer=:manufacturer', $stmt);
            $latestImagingSession=database::queryGetOne($stmt,array(':manufacturer'=>'Formulatrix'));
            if(!$latestImagingSession){ return $ret; }
            $fxImagingTaskId=$latestImagingSession['manufacturerdatabaseid'];
        } else {
            //Specific Fx imaging task
            $stmt=str_replace('[[CONDITION]]','ims.manufacturerdatabaseid=:id', $stmt);
            $latestImagingSession=database::queryGetOne($stmt,array(':id'=>$fxImagingTaskId));
            if(!$latestImagingSession){ return $ret; }
        }

        $ret['dateTime']=$latestImagingSession['imageddatetime'];
        $ret['fxImagingTaskId']=$fxImagingTaskId;

        $images=database::queryGetAll('
            SELECT SQL_CALC_FOUND_ROWS di.id
            FROM imagingsession AS ims, dropimage AS di
            WHERE di.imagingsessionid=ims.id
            AND ims.manufacturerdatabaseid=:fxid
        ', array(
            ':fxid'=>$fxImagingTaskId
        ));
        if($images){
            $ret['numImages']=$images['total'];
        }
        session::revokeAdmin();
        return $ret;
    }

    public static function createUser($params=array()){
        static::require('email',$params);
        static::require('fullName',$params);
        session::becomeAdmin();
        $user=user::getByProperty('email', $params['email']);
        if($user){
            if(1!=count($user['rows'])){
                throw new ServerException('More than one user with email '.$params['email']);
            }
            $user=$user['rows'][0];
        } else {
            $temporaryUsername='AUTO'.rand(10000000,99999999);
            while(user::getByName($temporaryUsername)){
                $temporaryUsername='AUTO'.rand(10000000,99999999);
            }
            $user=user::create(array(
                'fullname'=>$params['fullName'],
                'email'=>$params['email'],
                'name'=>$temporaryUsername,
                'password'=>'ChangeMe!',
                'isactive'=>0
            ));
            $user=$user['created'];
        }
        session::revokeAdmin();
        return $user;
    }

    public static function createPlateType($params){
        static::require('name',$params);
        $plateType=platetype::getByName($params['name']);
        if(!$plateType){
            static::require('rows',$params);
            static::require('columns',$params);
            static::require('subpositions',$params);
            session::becomeAdmin();
            $sharedProject=project::getByName(project::SHARED);
            if(!$sharedProject){
                throw new NotFoundException('No project called '.project::SHARED);
            }$dropMapping=array('','');
            for($i=1;$i<=$params['subpositions'];$i++){
                $dropMapping[0].=$i;
                $dropMapping[1].='R';
            }
            $dropMapping=implode(',', $dropMapping);
            $plateType=platetype::create(array(
                'name'=>$params['name'],
                'projectid'=>$sharedProject['id'],
                'rows'=>$params['rows'],
                'cols'=>$params['columns'],
                'subs'=>$params['subpositions'],
                'dropmapping'=>$dropMapping
            ));
            session::revokeAdmin();
            $plateType=$plateType['created'];
        }
        return $plateType;
    }

    public static function createPlate($params){
        static::require('scoringSystemName', $params);
        static::require('plateType', $params);
        static::require('owner', $params);
        static::require('barcode', $params);
        session::becomeAdmin();
        $plate=plate::getByName($params['barcode']);
        if(!$plate){
            $scoringSystem=crystalscoringsystem::getByName($params['scoringSystemName']);
            if(!$scoringSystem){
                throw new NotFoundException('No scoring system called '.$params['scoringSystemName']);
            }
            $plateType=static::createPlateType($params['plateType']);
            $owner=static::createUser($params['owner']);
            session::becomeAdmin(); //previous two creates revoked our admin status
            $project=project::getByName(project::DEFAULTPROJECT);
            $plate=plate::create(array(
                'name'=>$params['barcode'],
                'projectid'=>$project['id'],
                'ownerid'=>$owner['id'],
                'platetypeid'=>$plateType['id'],
                'crystalscoringsystemid'=>$scoringSystem['id']
            ));
            $plate=$plate['created'];
        }
        session::revokeAdmin();
        return $plate;
    }

    public static function createImager($params){
        static::require('name', $params);
        static::require('temperature', $params);
        $imagerName=$params['name'];
        if(""==$imagerName){
            $imagerName='Unknown '.$params['temperature'].' C imager';
        }
        $imager=imager::getByName($imagerName);
        if(!$imager){
            if(!preg_match('/^RI\d+-\d+$/', $imagerName)){
                throw new BadRequestException('Imager name does not match Formulatrix serial format RInn-nnnn');
            }
            //Formulatrix serials are typically RIxxx-nnnn, where X is the nominal capacity. We can't easily determine the
            //licensed capacity, which may be lower than the nominal capacity.
            $capacity=(int)(substr($imagerName, 2, 2-strpos('-', $imagerName)));
            session::becomeAdmin();
            imager::create(array(
                'name'=>$imagerName,
                'friendlyname'=>$imagerName,
                'manufacturer'=>'Formulatrix',
                'temperature'=>$params['temperature'],
                'platecapacity'=>$capacity,
                'alertlevel'=>floor(0.75*$capacity),
                'warninglevel'=>floor(0.9*$capacity),
            ));
            $imager=imager::getByName($imagerName);
            session::revertAdmin();
        }
        return $imager;
    }


    public static function createImagingSettings($params){
        /*
                {
                    "captureProfile":{
                        "name:"Test settings [[TIME]]",
                        "id":[[TIME]],
                        "currentVersionId":[[TIME]]0,
                        "settings":{
                            "LightPath":"Visible",
                            "TestSetting":"Test"
                        }
                    },
                    "imager":{
                        "name":"RI54-0039",
                        "temperature":20
                    }
                }
         */
        static::require('captureProfile',$params);
        static::require('name',$params['captureProfile']);
        static::require('id',$params['captureProfile']);
        static::require('currentVersionId',$params['captureProfile']);
        static::require('settings',$params['captureProfile']);
        static::require('imager',$params);
        static::require('name',$params['imager']);
        static::require('temperature',$params['imager']);
        session::becomeAdmin();
        $sharedProject=project::getByName(project::SHARED);
        $sharedProjectId=$sharedProject['id'];
        $imager=static::createImager($params['imager']);
        $imagerId=$imager['id'];
        $fxCaptureProfileName=$params['captureProfile']['name'];
        $fxCaptureProfileId=$params['captureProfile']['id'];
        $fxCaptureProfileVersionId=$params['captureProfile']['currentVersionId'];
        $ibImagingParameters=imagingparameters::getByProperties(array(
            'manufacturer'=>'Formulatrix',
            'manufacturerdatabaseid'=>$fxCaptureProfileId
        ));
        if($ibImagingParameters) {
            $ibImagingParametersId=$ibImagingParameters['rows'][0]['id'];
        } else {
            $ibImagingParameters=imagingparameters::create(array(
                'name'=>$params['captureProfile']['name'],
                'projectid'=>$sharedProjectId,
                'manufacturer'=>'Formulatrix',
                'manufacturerdatabaseid'=>$fxCaptureProfileId,
            ));
            $ibImagingParametersId=$ibImagingParameters['created']['id'];
        }
        $ibImagingParametersVersion=imagingparametersversion::getByProperties(array(
            'imagingparametersid'=>$ibImagingParametersId,
            'manufacturerdatabaseid'=>$fxCaptureProfileVersionId
        ));
        if($ibImagingParametersVersion){
            $ibImagingParametersVersion=$ibImagingParametersVersion['rows'][0];
            $ibImagingParametersVersionId=$ibImagingParametersVersion['id'];
        } else {
            $ibImagingParametersVersion=imagingparametersversion::create(array(
                'name'=>$params['captureProfile']['name'],
                'projectid'=>$sharedProjectId,
                'imagingparametersid'=>$ibImagingParametersId,
                'manufacturerdatabaseid'=>$fxCaptureProfileVersionId
            ));
            $ibImagingParametersVersionId=$ibImagingParametersVersion['created']['id'];
            imagingparameters::update($ibImagingParametersId, array(
                'currentversionid'=>$ibImagingParametersVersionId,
            ));
            foreach ($params['captureProfile']['settings'] as $setting=>$value){
                imagingsetting::create(array(
                    'settingname'=>$setting,
                    'projectid'=>$sharedProjectId,
                    'imagerid'=>$imagerId,
                    'imagingparametersversionid'=>$ibImagingParametersVersionId,
                    'name'=>'v'.$ibImagingParametersVersionId.'_robot'.$imagerId.'_'.$setting,
                    'settingvalue'=>$value
                ));
            }
        }
        imagingparametersversion::update($ibImagingParametersVersionId, array(
            'name'=>$params['captureProfile']['name'].'_v'.$ibImagingParametersVersionId
        ));
        imagingparameters::update($ibImagingParametersId, array(
            'currentversionid'=>$ibImagingParametersVersionId
        ));
        $ibImagingParameters=imagingparameters::getByProperties(array(
            'name'=>$fxCaptureProfileName,
            'manufacturer'=>'Formulatrix'
        ))["rows"][0];
        $ibImagingParameters['imagingparametersversion']=imagingparametersversion::getById($ibImagingParameters['currentversionid']);
        session::revertAdmin();
        return $ibImagingParameters;
    }

    public static function createImagingSession($params){
        static::require('imagingTaskId', $params);
        static::require('plateBarcode', $params);
        static::require('temperature', $params);
        static::require('imagerName', $params);
        static::require('imagedDateTime', $params);
        static::require('captureProfileVersionId', $params);
        session::becomeAdmin();
        $imagingParametersVersion=imagingparametersversion::getByProperties(array(
            'manufacturerdatabaseid'=>$params['captureProfileVersionId']
        ));
        if(!$imagingParametersVersion){
            throw new BadRequestException('No IceBear imagingparametersversion with manufacturerdatabaseid ['.$params['captureProfileVersionId'].']');
        }
        $imagingParametersVersion=$imagingParametersVersion['rows'][0];
        $imagingParametersVersionId=$imagingParametersVersion['id'];

        $imagingSession=imagingsession::getByProperties(array(
            'manufacturerdatabaseid'=>$params['imagingTaskId'],
            'imagingparametersversionid'=>$imagingParametersVersionId
        ));

        if($imagingSession){

            if(!$imagingSession['rows'] || 1!==count($imagingSession['rows'])){
                throw new ServerException('Found no/multiple imagingSession matches, should be 1');
            }
            $imagingSession=$imagingSession['rows'][0];

        } else {

            $imagingParameters=imagingparameters::getByProperties(array(
                'currentversionid'=>$imagingParametersVersionId,
                'manufacturer'=>'Formulatrix'
            ));
            if(!$imagingParameters){
                throw new BadRequestException('No IceBear imagingparameters with currentversionid ['.$imagingParametersVersionId.']');
            }
            $imagingParameters=$imagingParameters['rows'][0];
            $imagingParametersId=$imagingParameters['id'];
            $imagingParametersName=$imagingParameters['name'];
            $imagingParametersVersionId=$imagingParameters['currentversionid'];
            $captureProfileId=$imagingParameters['manufacturerdatabaseid'];

            //Subtract "captureProfileId" seconds to the imaged date/time, ensuring that the plate does not have two
            //imagingsessions at exactly the same time. (A Formulatrix ImagingTask can contain images from more than
            //one capture profile, which are split into different imagingsessions in IceBear.) We subtract because
            //"Imager defaults" is most likely lowest CaptureProfileId, and we want that to appear as the most recent.
            $imagedDateTime=str_replace('_',' ', $params['imagedDateTime']);
            $imagedDateTime=str_replace('T',' ', $imagedDateTime);
            try {
                $date=new DateTime($imagedDateTime);
            } catch (Exception $e){
                throw new BadRequestException('Bad date string in createImagingSession: '.$params['imagedDateTime']);
            }
            $date->sub(DateInterval::createFromDateString('+'.$captureProfileId.' seconds'));
            $imagedDateTime=date_format($date, 'Y-m-d H:i:s');

            $lightType='Visible';
            if(in_array(substr($imagingParametersName,0,3), array('UV ','UV-','UV_'))){
                $lightType='UV';
            } else {
                $settings=imagingsetting::getByProperty('imagingparametersversionid', $imagingParametersVersionId);
                if($settings){
                    foreach ($settings['rows'] as $setting){
                        if("LightType"==$setting['settingname'] || "LightPath"==$setting['settingname']){
                            $lightType=$setting['settingvalue'];
                        }
                    }
                }
            }

            $imager=imager::getByName($params['imagerName']);
            if(!$imager){
                throw new BadRequestException('No IceBear imager with name '.$params['imagerName']);
            }
            $plate=plate::getByName($params['plateBarcode']);
            if(!$plate){
                throw new BadRequestException('No IceBear plate with barcode '.$params['plateBarcode']);
            }
            $imagingSession=imagingsession::create(array(
                'plateid'=>$plate['id'],
                'projectid'=>$plate['projectid'],
                'imagerid'=>$imager['id'],
                'imagingparametersversionid'=>$imagingParametersVersionId,
                'manufacturerdatabaseid'=>$params['imagingTaskId'],
                'temperature'=>$params['temperature'],
                'imageddatetime'=>$imagedDateTime,
                'lighttype'=>$lightType,
                'name'=>$params['plateBarcode'].'_'.$imagedDateTime.'_profile'.$imagingParametersId
            ));
            $imagingSession=$imagingSession['created'];
        }
        $imagingSession['imager']=$imager;
        session::revertAdmin();
        return $imagingSession;
    }

    public static function createImages($params){
        static::require('images',$params);
        $ret=[];
        foreach($params['images'] as $image) {
            try{
                $ret[]=static::createImage($image);
            } catch (Exception $e){
                $ret[]=array('error'=>$e->getMessage());
            }
        }
        return $ret;
    }

    public static function createImage($params){
        static::require('iceBearImagingSessionId',$params);
        static::require('iceBearPlateId',$params);
        static::require('wellRowNumber',$params);
        static::require('wellColumnNumber',$params);
        static::require('wellDropNumber',$params);
        static::require('image',$params);
        static::require('thumb',$params);

        set_time_limit(300);

        session::becomeAdmin();

        $imagingSession=imagingsession::getById($params['iceBearImagingSessionId']);
        if(!$imagingSession){
            throw new BadRequestException('No IceBear imagingsession with ID '.$params['iceBearImagingSessionId']);
        }

        $plate=plate::getById($params['iceBearPlateId']);
        if(!$plate){
            throw new BadRequestException('No IceBear plate with ID '.$params['iceBearPlateId']);
        }
        $barcode=$plate['name'];
        $projectId=$plate['projectid'];

        $wellDrop=welldrop::getByPlateIdAndRowColumnDrop(
            $params['iceBearPlateId'],
            $params['wellRowNumber'],
            $params['wellColumnNumber'],
            $params['wellDropNumber']
        );
        if(!$wellDrop){
            throw new BadRequestException('No IceBear welldrop matching plateid '.$params['iceBearPlateId'].
                ' row '.$params['wellRowNumber'].' column '.$params['wellColumnNumber'].' drop '.$params['wellDropNumber']
            );
        }
        $wellDropId=$wellDrop['id'];

        $wellLabel=platewell::getWellLabel($params['wellRowNumber'], $params['wellColumnNumber']);
        $imageName=$barcode.'_'.$wellLabel.'.'.$params['wellDropNumber'].'_is'.$params['iceBearImagingSessionId'];
        $existing=dropimage::getByName($imageName);
        if($existing){
            return $existing;
        }

        $imageStore=rtrim(config::get('core_imagestore'),'/').'/';
        $imageFinalPath=substr($barcode,0,2).'/'.substr($barcode,0,3).'/'.$barcode.'/imagingsession'.$params['iceBearImagingSessionId'].'/';
        $thumbFinalPath=$imageFinalPath.'thumbs/';

        $random=bin2hex(random_bytes(24));
        $imageTempPath=$imageStore.$random.'_image';
        $thumbTempPath=$imageStore.$random.'_thumb';

        @file_put_contents($imageTempPath, base64_decode($params['image']));
        @file_put_contents($thumbTempPath, base64_decode($params['thumb']));

        if(!file_exists($imageTempPath) || !file_exists($imageStore.$random.'_thumb')){
            throw new ServerException('Could not write drop image/thumb to temporary storage');
        }
        $imageInfo=getimagesize($imageTempPath);
        $thumbInfo=getimagesize($thumbTempPath);
        if(!$imageInfo || !$thumbInfo){
            @unlink($imageTempPath);
            @unlink($thumbTempPath);
            throw new ServerException('Could not determine dimensions and type. Either image or thumb is corrupt.');
        }
        $pixelWidth=$imageInfo[0];
        $pixelHeight=$imageInfo[1];
        $imageType=$imageInfo[2];
        if(IMAGETYPE_JPEG==$imageType){
            $extension='.jpg';
        } else if(IMAGETYPE_PNG==$imageType){
            $extension='.png';
        } else {
            throw new BadRequestException("Image is not a JPEG or PNG");
        }

        if(!file_exists($imageStore.$thumbFinalPath) && !@mkdir($imageStore.$thumbFinalPath, 0755, true)){
            throw new ServerException('Could not create final storage directory for image/thumb: '.$imageStore.$thumbFinalPath);
        }

//        $wellLabel=platewell::getWellLabel($params['wellRowNumber'], $params['wellColumnNumber']);
        $filename=$wellLabel.'_'.$params['wellDropNumber'].$extension;
        $imageFinalPath.=$filename;
        $thumbFinalPath.=$filename;

        if(!@rename($imageTempPath, $imageStore.$imageFinalPath) || !@rename($thumbTempPath, $imageStore.$thumbFinalPath)){
            throw new ServerException('Could not move temporary image/thumb to final storage');
        }

//        $imageName=$barcode.'_'.$wellLabel.'.'.$params['wellDropNumber'].'_is'.$params['iceBearImagingSessionId'];

        $image=dropimage::create(array(
            'name'=>$imageName,
            'projectid'=>$projectId,
            'imagingsessionid'=>$params['iceBearImagingSessionId'],
            'plateid'=>$params['iceBearPlateId'],
            'welldropid'=>$wellDropId,
            'pixelwidth'=>$pixelWidth,
            'pixelheight'=>$pixelHeight,
            'micronsperpixelx'=>$params['micronsPerPixel'],
            'micronsperpixely'=>$params['micronsPerPixel'],
            'imagestorepath'=>$imageStore,
            'imagepath'=>$imageFinalPath,
            'thumbnailpath'=>$thumbFinalPath
        ));

        session::revertAdmin();

        return $image['created'];
    }


    public static function updateImagerInventories($params){
        static::require('imagers',$params);
        session::becomeAdmin();
        $sharedProject=project::getByName(project::SHARED);
        $updatedImagers=[];
        foreach($params['imagers'] as $fxImager){
            static::require('name',$fxImager);
            static::require('inventory',$fxImager);

            $imagerName=$fxImager["name"];
            $ibImager=imager::getByName($imagerName);
            if(!$ibImager){
                // This could happen when importing into a newly-installed IceBear. Just leave it, we'll
                // update its inventory when it's been created, after we import the first inspection from that imager.
                continue;
            }

            $active=[];
            $expired=[];

            // Set all plates with imager id to null locationid
            database::query('UPDATE plate SET locationid=NULL, nextinspectiontime=NULL, finalinspectiontime=NULL, inspectionsremaining=NULL
                        WHERE locationid=:imagerid', array(':imagerid'=>$ibImager['id']));

            // Set location, next/final inspection, and number of remaining inspections for all plates in the imager
            // (except where they don't exist in IceBear yet)
            $inventory=$fxImager['inventory'];
            foreach ($inventory as $fxPlate) {
                if(0==$fxPlate['InspectionsRemaining']){
                    $expired[]=$fxPlate;
                } else {
                    $active[]=$fxPlate;
                }

                $ibPlate=plate::getByName($fxPlate['Barcode']);
                if(!$ibPlate){
                    //Could happen if the Formulatrix side calls updateImagerInventories before trying to import inspections, or
                    //if some newly-inserted plates have not had their first inspection imported yet.
                    continue;
                }
                $parameters=array(
                    'locationid'=>$ibImager['id'],
                    'nextinspectiontime'=>$fxPlate['NextInspection'],
                    'finalinspectiontime'=>$fxPlate['FinalInspection'],
                    'inspectionsremaining'=>$fxPlate['InspectionsRemaining'],
                );
                plate::update($ibPlate['id'],$parameters);
            }

            //Update the imager's active/expired count
            imager::update($ibImager['id'], array(
                'platesactive'=>count($active),
                'platesexpired'=>count($expired)
            ));

            //Write the imager's active/expired plate count to the imager load log
            $date=gmdate("Y-m-d");
            $request=array(
                'projectid'=>$sharedProject['id'],
                'imagerid'=>$ibImager['id'],
                'loadingdate'=>$date,
                'platesactive'=>count($active),
                'platesexpired'=>count($expired),
            );
            imagerloadlog::log($request);

            $updatedImagers[]=$ibImager["name"];
        }

        session::revertAdmin();
        return array('updatedImagers'=>$updatedImagers);
    }

}
